@@ -11,35 +11,32 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 
-import os
-import sys
-import time
-import random
-import shutil
-import contextlib
-import tempfile
 import binascii
-import platform
-import select
+import contextlib
 import datetime
+import os
+import platform
+import random
+import select
+import shutil
+import sys
+import tempfile
+import time
 import unittest
-from unittest import mock
+from contextlib import ContextDecorator
 from io import BytesIO
-from subprocess import Popen, PIPE
+from subprocess import PIPE, Popen
+from unittest import mock
 
 from dateutil.tz import tzlocal
 
 import botocore.loaders
 import botocore.session
+from botocore import credentials, utils
 from botocore.awsrequest import AWSResponse
-from botocore.compat import (
-    parse_qs, six, urlparse, HAS_CRT
-)
-from botocore import utils
-from botocore import credentials
+from botocore.compat import HAS_CRT, parse_qs, six, urlparse
 from botocore.stub import Stubber
 
-
 _LOADER = botocore.loaders.Loader()
 
 
@@ -347,6 +344,7 @@ def _urlparse(url):
         url = url.decode('utf8')
     return urlparse(url)
 
+
 def assert_url_equal(url1, url2):
     parts1 = _urlparse(url1)
     parts2 = _urlparse(url2)
@@ -533,3 +531,32 @@ class StubbedSession(botocore.session.Session):
     def verify_stubs(self):
         for stub in self._client_stubs.values():
             stub.assert_no_pending_responses()
+
+
+class FreezeTime(ContextDecorator):
+    """
+    Context manager for mocking out datetime in arbitrary modules when creating
+    performing actions like signing which require point in time specificity.
+
+    :type module: module
+    :param module: reference to imported module to patch (e.g. botocore.auth.datetime)
+
+    :type date: datetime.datetime
+    :param date: datetime object specifying the output for utcnow()
+    """
+
+    def __init__(self, module, date=None):
+        if date is None:
+            date = datetime.datetime.utcnow()
+        self.date = date
+        self.datetime_patcher = mock.patch.object(
+            module, 'datetime',
+            mock.Mock(wraps=datetime.datetime)
+        )
+
+    def __enter__(self, *args, **kwargs):
+        mock = self.datetime_patcher.start()
+        mock.utcnow.return_value = self.date
+
+    def __exit__(self, *args, **kwargs):
+        self.datetime_patcher.stop()
